--[[
	Name: GridTargets
	Revision: $Revision: 82146 $
	Developed by: Julith @ Perenolde-EU (billgatesgut@web.de)
]]

--{{{ Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridTargets")

--}}}

L:RegisterTranslations("enUS", function()
	return {
		    ["GridTargets"] = true,
            ["mob health"] = true,
            ["mob target"] = true,
            ["mob aggro"] = true,
            ["Use class color"] = true,
            ["Color target based on class."] = true,
            ["Ignore self"] = true,
            ["Does not display if you are the target"] = true,
            ["Use threat color"] = true,
            ["Color health based on your threat on the mob."] = true,
            ["Tanking"] = true,
            ["Color for Tanking."] = true,
            ["Almost lost"] = true,
            ["Color for still tanking but almost lost."] = true,
            ["Lost aggro"] = true,
            ["Color for not tanking."] = true,
	}
end)

GridTargets = GridStatus:NewModule("GridTargets")
GridTargets.menuName = L["GridTargets"]


--{{{ AceDB defaults

GridTargets.defaultDB = {
	mobHealth = {
		enable = true,
		priority = 90,
		color = { r = 0.5, g = 0.5, b = 0.5, a = 1 },
        threatColor = { 
            [3] = { r = 0, g = 1, b = 0, a = 1 },
            [2] = { r = 0.5, g = 0.5, b = 0, a = 1 },
            [1] = { r = 1, g = 0, b = 0, a = 1 },
        },
        useThreatColor = true,
        ignoreSelf = true,
	},
    mobTarget = {
		enable = true,
		priority = 90,
		color = { r = 1, g = 0, b = 0, a = 1 },
        useClassColor = true,
        ignoreSelf = true,
	},
    updateInterval = 0.5
}

--}}}

local function getstatuscolor(status)
    if status == 0 then status = 1 end
    local color = GridTargets.db.profile.mobHealth.threatColor[status]
    return color.r, color.g, color.b, color.a
end

local function setstatuscolor(status, r, g, b, a)
    local color = GridTargets.db.profile.mobHealth.threatColor[status]
    color.r = r
    color.g = g
    color.b = b
    color.a = a or 1
end

--{{{ Options
local options = {
    [1] = {
        type = "color",
        name = L["Tanking"],
        desc = L["Color for Tanking."],
        order = 87,
        hasAlpha = true,
        get = function () return getstatuscolor(3) end,
        set = function (r, g, b, a) setstatuscolor(3, r, g, b, a) end,
    },
    [2] = {
        type = "color",
        name = L["Almost lost"],
        desc = L["Color for still tanking but almost lost."],
        order = 88,
        hasAlpha = true,
        get = function () return getstatuscolor(2) end,
        set = function (r, g, b, a) setstatuscolor(2, r, g, b, a) end,
    },
    [3] = {
        type = "color",
        name = L["Lost aggro"],
        desc = L["Color for not tanking."],
        order = 89,
        hasAlpha = true,
        get = function () return getstatuscolor(1) end,
        set = function (r, g, b, a) setstatuscolor(1, r, g, b, a) end,
    },
    ["useThreatColor"] = {
		type = "toggle",
		name = L["Use threat color"],
		desc = L["Color health based on your threat on the mob."],
		get = function ()
			      return GridTargets.db.profile.mobHealth.useThreatColor
		      end,
		set = function (v)
			      GridTargets.db.profile.mobHealth.useThreatColor = v
			      GridTargets:GMUpdate()
		      end,
	},
    ["ignoreSelf"] = {
		type = "toggle",
		name = L["Ignore self"],
		desc = L["Does not display if you are the target"],
		get = function ()
			      return GridTargets.db.profile.mobHealth.ignoreSelf
		      end,
		set = function (v)
			      GridTargets.db.profile.mobHealth.ignoreSelf = v
			      GridTargets:GMUpdate()
		      end,
	},
}

local targetOptions = {
    ["useClassColors"] = {
		type = "toggle",
		name = L["Use class color"],
		desc = L["Color target based on class."],
		get = function ()
			      return GridTargets.db.profile.mobTarget.useClassColor
		      end,
		set = function (v)
			      GridTargets.db.profile.mobTarget.useClassColor = v
			      GridTargets:GMUpdate()
		      end,
	},
    ["ignoreSelf"] = {
		type = "toggle",
		name = L["Ignore self"],
		desc = L["Does not display if you are the target"],
		get = function ()
			      return GridTargets.db.profile.mobTarget.ignoreSelf
		      end,
		set = function (v)
			      GridTargets.db.profile.mobTarget.ignoreSelf = v
			      GridTargets:GMUpdate()
		      end,
	},
}
--}}}

local settings

function GridTargets:OnInitialize()
	self.super.OnInitialize(self)
	self:RegisterStatus("mobHealth", L["mob health"], options, true)
    self:RegisterStatus("mobTarget", L["mob target"], targetOptions, true)
    settings = GridTargets.db.profile
end

function GridTargets:OnEnable()
    self:RegisterEvent("UNIT_TARGET", "Update")
    self:RegisterEvent("Grid_UnitJoined")
	self:RegisterEvent("Grid_UnitChanged")
    
    self:ScheduleRepeatingEvent("GMUpdate", "GMUpdate", settings.updateInterval)    
    self:RegisterEvent("GMUpdate")
end

function GridTargets:Grid_UnitJoined(unitGUID, unitid)
	self:Update(unitid)
end

function GridTargets:Grid_UnitChanged(unitGUID, unitid)
    self:Update(unitid)
end

function isMob(unitid)
    return UnitExists(unitid) and UnitIsEnemy("player", unitid) and not UnitIsDead(unitid)
end

function GridTargets:Update(unitid)
    local guid = UnitGUID(unitid)
    local target = unitid.."target"
    
    if not isMob(target) then
        self.core:SendStatusLost(guid, "mobHealth")
        self.core:SendStatusLost(guid, "mobTarget")
        return
    end

    local health = UnitHealth(target)
    local maxHealth = UnitHealthMax(target)
    health = maxHealth - health
    
    local status = UnitThreatSituation("player", target)
    
    if settings.mobHealth.ignoreSelf and status == 3 then
        self.core:SendStatusLost(guid, "mobHealth")
    else
        local col
        if settings.mobHealth.useThreatColor and status then
            col = {}
            col.r, col.g, col.b, col.a = getstatuscolor(status)
        else
            col = settings.mobHealth.color
        end

        self.core:SendStatusGained(guid, "mobHealth",
                        settings.mobHealth.priority,
                        nil,
                        col,
                        nil,
                        health, maxHealth)
    end
                    
    local targetsTarget = target.."target"
    if not UnitExists(targetsTarget) or settings.mobTarget.ignoreSelf and UnitIsUnit("player", targetsTarget) then
        self.core:SendStatusLost(guid, "mobTarget")
        return
    end
    
    local name = UnitName(targetsTarget)
    local ttGuid = UnitGUID(targetsTarget)
    
    self.core:SendStatusGained(guid, "mobTarget",
				    settings.mobTarget.priority,
				    nil,
				    (settings.mobTarget.useClassColor and 
                     self.core:UnitColor(ttGuid) or
                     settings.mobTarget.color),
                    name)
end

function GridTargets:GMUpdate()
    for guid, unitid in GridRoster:IterateRoster() do
		self:Update(unitid)
	end
end

function GridTargets:UNIT_TARGET(unitid)    
    self:Update(unitid)
end


